Atmosphere

Struct Atmosphere 

pub struct Atmosphere {
    pub inner_radius: f32,
    pub outer_radius: f32,
    pub ground_albedo: Vec3,
    pub medium: Handle<ScatteringMedium>,
}
Atmosphere for one planet. The entity’s GlobalTransform is the planet center in world space.

Add AtmosphereSettings to each 3D camera that should use it, the nearest atmosphere is used for rendering.

If GlobalTransform is still Default when this component is first added, it is placed radius units directly below the origin on the Y axis, so that the planet’s normal is roughly Vec3::Y around the origin, likely where your camera/scene is located. Unless you’re making a game set in space, this is probably what you want. Otherwise, feel free to override this default by setting a transform manually.

The scale on GlobalTransform rescales the planet in world space. Tune it with the radius offset when your scene uses other units, like kilometer-sized scenes.

Fields

inner_radius: f32

Radius of the planet

units: m

outer_radius: f32

Radius at which we consider the atmosphere to ‘end’ for our calculations (from center of planet)

units: m

ground_albedo: Vec3

An approximation of the average albedo (or color, roughly) of the planet’s surface. This is used when calculating multiscattering.

units: N/A

medium: Handle<ScatteringMedium>

A handle to a ScatteringMedium, which describes the substance of the atmosphere and how it scatters light.

Implementations

Source

impl Atmosphere

pub fn earth(medium: Handle<ScatteringMedium>) -> Atmosphere

An atmosphere like that of earth. Use this with a ScatteringMedium::earth handle.

pub fn mars(medium: Handle<ScatteringMedium>) -> Atmosphere

Martian atmosphere; use this with a ScatteringMedium::mars handle.

Mean radius 3389.50 ± 0.2 km Seidelmann et al. 2007, Table 4.