pub struct ScatteringMedium {
pub label: Option<Cow<'static, str>>,
pub falloff_resolution: u32,
pub phase_resolution: u32,
pub terms: SmallVec<[ScatteringTerm; 1]>,
}Expand description
An asset that defines how a material scatters light.
In order to calculate how light passes through a medium, you need three pieces of information:
- how much light the medium absorbs per unit length
- how much light the medium scatters per unit length
- what directions the medium is likely to scatter light in.
The first two are fairly simple, and are sometimes referred to together (accurately enough) as the medium’s optical density.
The last, defined by a phase function, is the most important in creating
the look of a medium. Our brains are very good at noticing (if unconsciously)
that a dust storm scatters light differently than a rain cloud, for example.
See the docs on PhaseFunction for more info.
In reality, media are often composed of multiple elements that scatter light
independently, for Earth’s atmosphere is composed of the gas itself, but also
suspended dust and particulate. These each scatter light differently, and are
distributed in different amounts at different altitudes. In a ScatteringMedium,
these are each represented by a ScatteringTerm
Technical Details
A ScatteringMedium is represented on the GPU by a set of two LUTs, which
are re-created every time the asset is modified. See the docs on
bevy_pbr::GpuScatteringMedium for more info.
Fields
label: Option<Cow<'static, str>>An optional label for the medium, used when creating the LUTs on the GPU.
falloff_resolution: u32The resolution at which to sample the falloff distribution of each scattering term. Custom or more detailed distributions may benefit from a higher value, at the cost of more memory use.
phase_resolution: u32The resolution at which to sample the phase function of each scattering term. Custom or more detailed phase functions may benefit from a higher value, at the cost of more memory use.
terms: SmallVec<[ScatteringTerm; 1]>The list of ScatteringTerms that compose this ScatteringMedium
Implementations
Sourceimpl ScatteringMedium
impl ScatteringMedium
Sourcepub fn new(
falloff_resolution: u32,
phase_resolution: u32,
terms: impl IntoIterator<Item = ScatteringTerm>,
) -> ScatteringMedium
pub fn new( falloff_resolution: u32, phase_resolution: u32, terms: impl IntoIterator<Item = ScatteringTerm>, ) -> ScatteringMedium
Returns a scattering medium with a default label and the specified scattering terms.
Sourcepub fn with_label(self, label: impl Into<Cow<'static, str>>) -> ScatteringMedium
pub fn with_label(self, label: impl Into<Cow<'static, str>>) -> ScatteringMedium
Consumes and returns this scattering medium with a new label.
Sourcepub fn with_density_multiplier(self, multiplier: f32) -> ScatteringMedium
pub fn with_density_multiplier(self, multiplier: f32) -> ScatteringMedium
Consumes and returns this scattering medium with each scattering terms’
densities multiplied by multiplier.
Sourcepub fn earth(falloff_resolution: u32, phase_resolution: u32) -> ScatteringMedium
pub fn earth(falloff_resolution: u32, phase_resolution: u32) -> ScatteringMedium
Returns a scattering medium representing an earth atmosphere.
Uses physically-based scale heights from Earth’s atmosphere, assuming a 60 km atmosphere height:
- Rayleigh (molecular) scattering: 8 km scale height
- Mie (aerosol) scattering: 1.2 km scale height
Sourcepub fn mars(
falloff_resolution: u32,
phase_resolution: u32,
dust_phase: Handle<Image>,
) -> ScatteringMedium
pub fn mars( falloff_resolution: u32, phase_resolution: u32, dust_phase: Handle<Image>, ) -> ScatteringMedium
Returns a scattering medium representing a Martian atmosphere Schneegans et al. 2024.
Constituents:
- Rayleigh: carbon dioxide
- Dust: wavelength-dependent Mie phase, double-exponential density
Requires an Nx1 chromatic phase texture for the dust term.